import requests
from PIL import Image
from io import BytesIO
import time
import cv2
from skimage.metrics import structural_similarity as compare_ssim

import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.application import MIMEApplication
from os.path import basename

# Serverdaten
smtpServer = "smtp.web.de"
smtpPort = 587
# Zugangsdaten
username = "E-Mail Adresse"
password = "Passwort"

# Sender & Empfänger
sender = "Absender E-Mail"
reciever = "Empfänger E-Mail"

# IP-Adresse der ESP32-CAM
espCamIpAddress = 'http://192.168.178.33'
url = espCamIpAddress + '/capture'

# Verzeichnis für die Ablage der Dateien
outputFolder = 'data/'
# Dateiendung
outputFileExt = '.jpg'

"""
Index   | Auflösung
13      | 1600 x 1200
12      | 1280 x 1024
11      | 1280 x 720
10      | 1024 x 768
9       | 800 x 600
8       | 640 x 480
7       | 480 x 320
6       | 400 x 296
5       | 320 x 240
4       | 240 x 240
3       | 240 x 176
2       | 176 x 144
1       | 160 x 120
0       | 96 x 96
"""

# dict mit den Auflösungen und den Indexen
resolutions = (
    {"index": 0, "width": 96, "height": 96},
    {"index": 1, "width": 160, "height": 120},
    {"index": 2, "width": 176, "height": 144},
    {"index": 3, "width": 240, "height": 176},
    {"index": 4, "width": 240, "height": 240},
    {"index": 5, "width": 320, "height": 240},
    {"index": 6, "width": 400, "height": 296},
    {"index": 7, "width": 480, "height": 320},
    {"index": 8, "width": 640, "height": 480},
    {"index": 9, "width": 800, "height": 600},
    {"index": 10, "width": 1024, "height": 768},
    {"index": 11, "width": 1280, "height": 720},
    {"index": 12, "width": 1280, "height": 1024},
    {"index": 13, "width": 1600, "height": 1200}
)


def setResolution(imageResolution):
    url = espCamIpAddress + '/control?var=framesize&val=' + str(imageResolution['index'])
    r = requests.get(url)
    if r.status_code > 200:
        print("Fehler ", str(r.status_code), " bei setzen der Auflösung an der ESP32 CAM!")
    time.sleep(1)


def takePicture():
    setResolution(resolutions[9])
    r = requests.get(url)
    img = Image.open(BytesIO(r.content))
    timestamp = round(time.time())
    outputfilename = outputFolder + str(timestamp) + outputFileExt
    img.save(outputfilename)
    return outputfilename

def sendMail(filename):
    # Betreff & Inhalt
    subject = "Es wurde eine Bewegung erkannt!"
    body = "Es wurde eine Bewegung erkannt, ein großes Bild ist im Anhang dieser E-Mail."
    # Message Objekt für die E-Mail
    # später kann an dieses Objekt eine
    # oder mehrere Dateien angehängt
    # werden.
    msg = MIMEMultipart()
    msg['Subject'] = subject
    msg['From'] = sender
    msg['To'] = reciever
    msg.attach(MIMEText(body, 'plain'))

    with open(filename, "rb") as file:
        part = MIMEApplication(
            file.read(),
            Name=basename(filename)
        )
        part['Content-Disposition'] = 'attachment; filename="%s"' % basename(filename)
        msg.attach(part)

    # Erzeugen einer Mail Session
    smtpObj = smtplib.SMTP(smtpServer, smtpPort)
    # Debuginformationen auf der Konsole ausgeben
    smtpObj.set_debuglevel(1)
    # Wenn der Server eine Authentifizierung benötigt dann...
    smtpObj.starttls()
    smtpObj.login(username, password)
    # absenden der E-Mail
    smtpObj.sendmail(sender, reciever, msg.as_string())
    smtpObj.close()

def main():
    global imageResolution

    print("Take first image.")
    first_image = takePicture()
    print("Wait 5 seconds.")
    time.sleep(5);
    print("Take second image.")
    second_image = takePicture()

    image1 = cv2.cvtColor(cv2.imread(first_image), cv2.COLOR_BGR2GRAY)
    image2 = cv2.cvtColor(cv2.imread(takePicture()), cv2.COLOR_BGR2GRAY)

    (score, diff) = compare_ssim(image1, image2, full=True)
    diff = (diff * 255).astype("uint8")
    print("SSIM: {}".format(score))

    if score < 0.8:
        setResolution(resolutions[13])
        filename = takePicture()
        sendMail(filename)

if __name__ == "__main__":
    main()
